
import qdrant_client
from qdrant_client import QdrantClient, models
from llama_index.core.postprocessor import SimilarityPostprocessor
from llama_index.vector_stores.qdrant import QdrantVectorStore
from llama_index.core import VectorStoreIndex
from llama_index.core import Document
from .embedding_model import VolcanoEmbedding
from digital_life import logger

from llama_index.core.vector_stores import (
    MetadataFilter,
    MetadataFilters,
    FilterOperator,
)

import os


class EmbeddingPool:
    def __init__(self):
        self.reload()

    def create_collection(
        self, collection_name: str, vector_dimension: str = 2560
    ):
        distance_metric = models.Distance.COSINE  # 使用余弦相似度

        # 2. 定义 Collection 参数

        client = qdrant_client.QdrantClient(
            host=os.getenv("host", "localhost"),
            port=os.getenv("port", 6333) 
        )

        # 3. 创建 Collection (推荐使用 recreate_collection)
        try:
            client.recreate_collection(
                collection_name=collection_name,
                vectors_config=models.VectorParams(
                    size=vector_dimension, distance=distance_metric
                ),
            )
        except Exception as e:
            raise Exception("创建collection失败") from e
        finally:
            client.close()

    def reload(self):
        logger.info("reload")
        # 默认仓库要被创建的
        self.postprocess = SimilarityPostprocessor(
            similarity_cutoff=float(os.getenv("similarity_cutoff", 0.5))
        )
        client = qdrant_client.QdrantClient(
            host=os.getenv("host", "localhost"), 
            port=os.getenv("port", 6333) 
        )
        vector_store = QdrantVectorStore(
            client=client, collection_name=os.getenv("collection_name", "diglife")
        )
        self.embed_model = VolcanoEmbedding(
            model_name=os.getenv("model_name", "doubao-embedding-text-240715"),
            api_key=os.getenv("api_key", ""),
        )
        self.index = VectorStoreIndex.from_vector_store(
            vector_store, embed_model=self.embed_model
        )
        """
            # TODO add more operators
            EQ = "=="  # default operator (string, int, float)
            GT = ">"  # greater than (int, float)
            LT = "<"  # less than (int, float)
            NE = "!="  # not equal to (string, int, float)
            GTE = ">="  # greater than or equal to (int, float)
            LTE = "<="  # less than or equal to (int, float)
            IN = "in"  # In array (string or number)
            NIN = "nin"  # Not in array (string or number)
            ANY = "any"  # Contains any (array of strings)
            ALL = "all"  # Contains all (array of strings)
            TEXT_MATCH = "text_match"  # full text match (allows you to search for a specific substring, token or phrase within the text field)
            TEXT_MATCH_INSENSITIVE = (
                "text_match_insensitive"  # full text match (case insensitive)
            )
            CONTAINS = "contains"  # metadata array contains value (string or number)
            IS_EMPTY = "is_empty"  # the field is not exist or empty (null or empty array)

        """
        filters_bac = MetadataFilters(
            filters=[MetadataFilter(key="type", value=1, operator=FilterOperator.LTE)]
        )

        self.retriver_bac = self.index.as_retriever(
            filters=filters_bac, similarity_top_k=int(os.getenv("similarity_top_k", 10))
        )

        filters_figure_person = MetadataFilters(
            filters=[MetadataFilter(key="type", value=2, operator=FilterOperator.EQ)]
        )

        self.retriver_figure_person = self.index.as_retriever(
            filters=filters_figure_person,
            similarity_top_k=int(os.getenv("similarity_top_k", 10)),
        )


    def update(self, text: str, id: str, type: int) -> str:
        doc = Document(
            text=text,
            id_=id,
            metadata={"type": type, "id": id},
            excluded_embed_metadata_keys=["type", "id"],
        )
        self.index.update(document=doc)

    def delete(self, id: str):
        logger.info("delete")
        # 2025-09-18 10:37:32,238 - 警告 - llama_index.core.indices.base - base：310 - delete() 方法现已弃用，请使用 delete_ref_doc() 删除已导入的文档和节点，或使用 delete_nodes 删除节点列表。如需异步实现，请使用 delete_ref_docs() 。
        self.index.delete(doc_id=id)

    def search_bac(self, query: str) -> str:
        results = self.retriver_bac.retrieve(query)
        output = []
        for i, result in enumerate(results):
            dct = result.node.metadata
            dct.update({"order": i})
            output.append(dct)

        return output

    def search_figure_person(self, query: str) -> str:
        results = self.retriver_figure_person.retrieve(query)
        output = []
        for i, result in enumerate(results):
            dct = result.node.metadata
            dct.update({"order": i})
            output.append(dct)

        return output





import json
from pro_craft import AsyncIntel

import os
from typing import Dict, Any
from digital_life.core.recommended.redis_ import get_redis_client, store_with_expiration, get_value
from digital_life.utils import memoryCards2str, extract_article, extract_json
from digital_life.models import BiographyRequest, BiographyResult, Extract_Person,Extract_Place, Biography_Free, ContentVer
from digital_life import logger
from digital_life import super_log
import asyncio
import httpx
import uuid
import re
from pydantic import BaseModel, Field, model_validator, field_validator, RootModel


async def aget_(url = ""):
    async with httpx.AsyncClient() as client:
        try:
            response = await client.get(url)
            response.raise_for_status()  # 如果状态码是 4xx 或 5xx，会抛出 HTTPStatusError 异常
            
            print(f"Status Code: {response.status_code}")
            print(f"Response Body: {response.json()}") # 假设返回的是 JSON
            return response.json()
        except httpx.HTTPStatusError as e:
            print(f"HTTP error occurred: {e.response.status_code} - {e.response.text}")
        except httpx.RequestError as e:
            print(f"An error occurred while requesting {e.request.url!r}: {e}")
        except Exception as e:
            print(f"An unexpected error occurred: {e}")
    return None


def extract_from_text(text: str):
    matches = []
    for match in re.finditer(r'!\[\]\(([^)]+)\)', text):
        url = match.group(1).strip()
        position = match.start()
        matches.append((url, position))
    return matches



user_callback_url = os.getenv("user_callback_url")

# TODO 后续使用redis 进行任务队列设计

class BiographGenerateError(Exception):
    pass

def remove_urls_from_text(text: str) -> str:
    """
    检测文本中的 Markdown 格式图片链接 (![]()) 并将其剔除。

    Args:
        text: 待处理的字符串。

    Returns:
        剔除了 Markdown 格式图片链接的字符串。
    """
    # 使用 re.sub 替换所有匹配的模式为空字符串
    # r'!\[\]\([^)]+\)' 匹配 ![]() 结构，其中括号内的内容是 URL
    cleaned_text = re.sub(r'!\[\]\([^)]+\)', '', text)
    return cleaned_text


class BiographyGenerate:
    def __init__(self,inference_save_case = False,model_name = "doubao-1-5-pro-256k-250115"):
        self.inters = AsyncIntel(model_name = model_name)
        self.inference_save_case = inference_save_case
        # ArkAdapter
        self.biograph_redis = get_redis_client(username = os.getenv("redis_username"), 
                                             password = os.getenv("redis_password"), 
                                             host = os.getenv("redis_host"), 
                                             port = os.getenv("redis_port"),
                                             db = 22)

    def _split_into_chunks(self,my_list, chunk_size=5):
        """
        使用列表推导式将列表分割成大小为 chunk_size 的块。
        """
        return [
            my_list[i : i + chunk_size] for i in range(0, len(my_list), chunk_size)
        ]

    async def amaterial_generate(self, vitae: str, memory_cards: list[str]) -> str:
        """
        素材整理
        vitae : 简历
        memory_cards : 记忆卡片们
        0085 素材整理
        0082 素材增量生成
        """
        try:
            # --- 示例 ---
            chunks = self._split_into_chunks(memory_cards, chunk_size=2)

            material = ""
            for i, chunk in enumerate(chunks):
                chunk = json.dumps(chunk, ensure_ascii=False)
                if i == 0:
                    output_format = ""
                    # 素材整理初始
                    material = await self.inters.intellect(input_data=vitae + chunk,
                                        output_format=output_format,
                                        prompt_id ="biograph_material_init",
                                        version = None,
                                        inference_save_case = self.inference_save_case,
                                        )
                else:
                    # 素材增量生成
                    output_format = ""
                    material = await self.inters.intellect(input_data=material,
                                        output_format=output_format,
                                        prompt_id ="biograph_material_add",
                                        version = None,
                                        inference_save_case = self.inference_save_case,
                                        )
            super_log(material,'material_generate-material')
        except Exception as e:
            raise BiographGenerateError(f"素材整理出错 {e}") from e

        return material

    async def aoutline_generate(self, material: str) -> str:
        """
        0084 大纲生成
         #TODO 由于output_format 太过复杂导致无法使用pydantic
        """
        output_format = """
输出格式
```json
{
    "预章" : [
        {
            "chapter_number": "-",
            "title": "山村童年的自由奔跑",
            "topic": "第三人称。本章将追溯张三在1980年代末至1990年代初，辽宁葫芦岛小村庄的童年岁月。描绘乡村的淳朴生活、日出而作日落而息的节奏，以及他作为“山里的野孩子”与自然亲密接触的自由时光。这是张三生命中“第一起”的萌芽，奠定了他与自然连接的性格底色和对规律生活的最初认知。"
        },
    ]
    "第一部 童年与自然启蒙": [
        {
            "chapter_number": "第一章",
            "title": "山村童年的自由奔跑",
            "topic": "第三人称。本章将追溯张三在1980年代末至1990年代初，辽宁葫芦岛小村庄的童年岁月。描绘乡村的淳朴生活、日出而作日落而息的节奏，以及他作为“山里的野孩子”与自然亲密接触的自由时光。这是张三生命中“第一起”的萌芽，奠定了他与自然连接的性格底色和对规律生活的最初认知。"
        },
        {
            "chapter_number": "第二章",
            "title": "漫天繁星的宇宙初识",
            "topic": "第一人称。深入探究张三童年时期对夜晚星空的深刻记忆和感受。描述“漫天繁星”如何在他幼小的心灵中种下对宇宙和未知世界的好奇种子，成为他最早的“觉醒点”，预示其未来对知识的无尽求索。这是他“第一起”中重要的情感与思想铺垫。"
        }
    ]
    ...
    "尾章" : [
        {
            "chapter_number": "-",
            "title": "山村童年的自由奔跑",
            "topic": "第三人称。本章将追溯张三在1980年代末至1990年代初，辽宁葫芦岛小村庄的童年岁月。描绘乡村的淳朴生活、日出而作日落而息的节奏，以及他作为“山里的野孩子”与自然亲密接触的自由时光。这是张三生命中“第一起”的萌芽，奠定了他与自然连接的性格底色和对规律生活的最初认知。"
        },
    ]
}
```
"""
        
        try:
            outline_origin = await self.inters.intellect(input_data=material,
                                                output_format=output_format,
                                                prompt_id ="biograph-outline",
                                                version = None,
                                                inference_save_case=self.inference_save_case)
            outline = extract_json(outline_origin)
            result = json.loads(outline)

        except Exception as e:
            raise Exception(f"0084 传记大纲生成失败: {e}")
        return result
    
    async def title_generate(self, outline: dict) -> str:
        """
        0085 传记标题生成
        """
        class BiographPaidTitle(BaseModel):
            title: str = Field(..., description="传记标题")

        result = await self.inters.intellect_format(
            input_data=outline,
            prompt_id ="biograph-paid-title",
            OutputFormat=BiographPaidTitle,
            ExtraFormats=[],
            inference_save_case=self.inference_save_case
        )
        return result.get('title')

    async def agener_biography_brief(self, outline: dict) -> str:
        """
        0083 传记简介
        """
        # result = await self.inters.intellect_remove_format(
        #     input_data = json.dumps(outline, ensure_ascii=False),
        #     prompt_id = "0083",
        #     version = None,
        #     inference_save_case=self.inference_save_case,
        #     OutputFormat = ContentVer,
        # )
        output_format = ""
        result = await self.inters.intellect(input_data=json.dumps(outline, ensure_ascii=False),
                                    output_format=output_format,
                                    prompt_id ="biograph-brief",
                                    version = None,
                                    inference_save_case = self.inference_save_case,
                                    )

        result = extract_json(result)
        result = result.replace('"content":','')
        result = remove_urls_from_text(result)
        return result

    async def extract_person_name(self, bio_chunk: str):
        """0087 提取人名"""

        result = await self.inters.intellect(input_data=bio_chunk,
                                    output_format="",
                                    prompt_id ="biograph-extract-person-name",
                                    version = None,
                                    inference_save_case = self.inference_save_case,
                                    )
        result = json.loads(extract_json(result))
        result = result.get("content")
        return result

    async def extract_person_place(self, bio_chunk: str):
        """0086 提取地名"""

        # result = await self.inters.intellect_remove_format(
        #     input_data = bio_chunk,
        #     prompt_id = "0086",
        #     version = None,
        #     inference_save_case=self.inference_save_case,
        #     OutputFormat = Extract_Place,
        # )

        output_format = ""
        result = await self.inters.intellect(input_data=bio_chunk,
                                    output_format=output_format,
                                    prompt_id ="biograph-extract-place",
                                    version = None,
                                    inference_save_case = self.inference_save_case,
                                    )
        result = json.loads(extract_json(result))
        result = result.get("content")
        return result

    async def awrite_chapter(
        self,
        chapter,
        master="",
        material="",
        outline: dict = {},
        suggest_number_words=3000,
    ):
        created_material = ""
        try:
            # 0080 prompt_get_infos 0080 从素材中抽取必要撰写素材内容 biograph-extract-material
            # 0081 prompt_base  0081
            # TODO 大量的format 怎么办
            
            # material = await self.inters.intellect_remove_format(
            #     input_data = {
            #                     "material": material,
            #                     "frame": json.dumps(outline,ensure_ascii=False),
            #                     "Requirements for Chapter Writing": json.dumps(chapter,ensure_ascii=False)
            #                 },
            #     prompt_id = "0080",
            #     version = None,
            #     inference_save_case=self.inference_save_case,
            #     OutputFormat = ContentVer,
            #         )
            
            output_format = """"""
            try:
                material = await self.inters.intellect(input_data={
                                                                    "material": material,
                                                                    "frame": json.dumps(outline,ensure_ascii=False),
                                                                    "Requirements for Chapter Writing": json.dumps(chapter,ensure_ascii=False)
                                                                },
                                    output_format=output_format,
                                    prompt_id ="biograph-extract-material",
                                    version = None,
                                    inference_save_case = self.inference_save_case,
                                    )
            except Exception as e:
                raise Exception(f'素材收拾的时候报错 {e}')
            

            try:
                output_format = """"""
                article = await self.inters.intellect(input_data = {
                                                                "目标人物": master,
                                                                "章节名称": chapter.get("chapter_number") + "   " + chapter.get("title"),
                                                                "目标字数范围":suggest_number_words,
                                                                "核心主题": chapter.get("topic"),
                                                                "素材":material,
                                                            },
                                            output_format=output_format,
                                            prompt_id ="biograph-writer",
                                            version = None,
                                            inference_save_case = self.inference_save_case,
                                            )
            except Exception as e:
                raise Exception(f'这是在生成文章时候报错 {e}')
            try:
                chapter_name = await self.extract_person_name(article)
                chapter_place = await self.extract_person_place(article)
            except Exception as e:
                raise Exception(f'提取人名地名报错 {e}')
            try:
                # assert isinstance(chapter_name["content"], list)
                # assert isinstance(chapter_place["content"], list)
                1 == 1
            except Exception as e:
                raise Exception(f'断言出错 {e}')
            # a = {
            #                                     "article": article,
            #                                     "素材":material.get("content"),
            #                                             }
            # article = await self.inters.intellect_remove(
            #                             input_data = {
            #                                     "article": article,
            #                                     "素材":material.get("content"),
            #                                             },
            #                             output_format=output_format,
            #                             prompt_id ="0079",
            #                             version = None,
            #                             inference_save_case = self.inference_save_case,
            #                             )

            return {
                "chapter_number": chapter.get("chapter_number"),
                "article": article,
                "material": material,
                "created_material": created_material,
                "chapter_name": chapter_name,
                "chapter_place": chapter_place,
            }

        except Exception as e:
            print(f"Error processing chapter {chapter.get('chapter_number')}: {e}")

            return {
                "chapter_number": chapter.get("chapter_number"),
                "article": "",
                "material": "material",
                "created_material": "created_material",
                "chapter_name": "chapter_name",
                "chapter_place": "chapter_place",
            }

    async def agenerate_biography_free(
        self, user_name: str, vitae: str, memory_cards: list[dict]
    ):
        memoryCards_str, _ = memoryCards2str(memory_cards)
        result = await self.inters.intellect_format(
            input_data = f"{user_name},{vitae},{memoryCards_str}",
            prompt_id = "biograph-free-writer",# biograph-free-writer 0095
            version = None,
            inference_save_case=self.inference_save_case,
            OutputFormat = Biography_Free,
        )

        return result

    async def _generate_biography(self,task_id: str, 
                                  memory_cards: str,
                                  vitae: str,
                                  user_name: str):

        task = {
            "task_id": task_id,
            "status": "PENDING",
            "biography_title": None,
            "biography_brief": None,
            "biography_json": None,
            "biography_name": None,
            "biography_place": None,
            "error_message": None,
            "progress": 0.0,
            "request_data": "",  # 存储请求数据以备后续使用
        }
        task["status"] = "PROCESSING"
        task["progress"] = 0.1

        try:
            
            # 素材整理
            material = await self.amaterial_generate(
                vitae=vitae, memory_cards=memory_cards
            )
            task["progress"] = 0.2
            task["material"] = material
            store_with_expiration(self.biograph_redis, task_id, task, 3600) 

        
            # 生成大纲
            outline = await self.aoutline_generate(material)
            task["progress"] = 0.3
            task["outline"] = outline
            store_with_expiration(self.biograph_redis, task_id, task, 3600) 


            # 生成标题
            title = await self.title_generate(outline)
            task["progress"] = 0.4
            task["biography_title"] = title
            store_with_expiration(self.biograph_redis, task_id, task, 3600) 

            

            # 生成传记简介
            brief = await self.agener_biography_brief(outline)
            task["biography_brief"] = brief
            task["progress"] = 0.5
            store_with_expiration(self.biograph_redis, task_id, task, 3600) 


            # 生成传记正文  

            biography_json = {}
            biography_name = []
            biography_place = []

            tasks = []
            for part, chapters in outline.items():
                for chapter in chapters:
                    logger.info(
                        f"Creating task for chapter: {chapter.get('chapter_number')} {chapter.get('title')}"
                    )
                    tasks.append(
                        self.awrite_chapter(
                            chapter,
                            master=user_name,
                            material=material,
                            outline=outline,
                        )
                    )
            results = await asyncio.gather(*tasks, return_exceptions=False)
            
            # 后处理拼接
            for part, chapters in outline.items():
                biography_json[part] = []
                for chapter in chapters:
                    chapter_number = chapter.get("chapter_number")
                    for x in results:
                        if x.get("chapter_number") == chapter_number:
                            biography_json[part].append(x.get("article"))
                            biography_name += x.get("chapter_name")
                            biography_place += x.get("chapter_place")

            assert isinstance(biography_json, dict)
            assert isinstance(biography_name, list)
            assert isinstance(biography_place, list)

            biography_name = list(set(biography_name))
            biography_place = list(set(biography_place))
            task["biography_json"] = biography_json
            task["biography_name"] = biography_name
            task["biography_place"] = biography_place
            task["status"] = "COMPLETED"
            task["progress"] = 1.0


            biography_callback_url_success = user_callback_url + f'/api/inner/notifyBiographyStatus?generateTaskId={task_id}&status=1'
            store_with_expiration(self.biograph_redis, task_id, task, 3600) 
            await aget_(url = biography_callback_url_success)


        except Exception as e:
            task["status"] = "FAILED"
            task["error_message"] = str(e)
            task["progress"] = 1.0
            biography_callback_url_failed = user_callback_url + f'/api/inner/notifyBiographyStatus?generateTaskId={task_id}&status=0'
            store_with_expiration(self.biograph_redis, task_id, task, 3600) 

            await aget_(url = biography_callback_url_failed)



import time
import uuid
from typing import List, Optional, Dict, Union, Literal,Annotated
from pydantic import BaseModel, Field, model_validator


class ImageUrl(BaseModel):
    """定义了 image_url 字段的内部结构"""
    url: str

class TextPart(BaseModel):
    """代表类型为 'text' 的内容部分"""
    type: Literal['text']
    text: str

class ImagePart(BaseModel):
    """代表类型为 'image_url' 的内容部分"""
    type: Literal['image_url']
    image_url: ImageUrl

# 使用 "Tagged Union" 来让 Pydantic 根据 'type' 字段自动选择正确的模型
ContentPart = Annotated[
    Union[TextPart, ImagePart],
    Field(discriminator='type')
]

# --- 你的 ChatMessage 和 ChatCompletionRequest 模型 (修改 `content` 字段) ---

class ChatMessage(BaseModel):
    """
    更新后的 ChatMessage，其 content 字段可以接受 str 或 List[ContentPart]，
    并在内部统一为 List[ContentPart]。
    """
    role: Literal["system", "user", "assistant", "tool"]
    # content 字段现在可以接受 str 或 List[ContentPart]
    content: Optional[Union[str, List[ContentPart]]] = None
    # tool_calls: Optional[...] # Add if you support tool/function calling
    # tool_call_id: Optional[...] # Add if you support tool/function calling

    @model_validator(mode='after')
    def normalize_content(self) -> 'ChatMessage':
        """
        验证后标准化 content 字段：
        - 将 str 类型的 content 转换为 [TextPart]。
        - 合并连续的 TextPart。
        """
        if self.content is None:
            return self

        new_content: List[ContentPart] = []
        if isinstance(self.content, str):
            if self.content: # 避免创建空的 text part
                new_content.append(TextPart(type='text', text=self.content))
        else: # self.content is List[ContentPart]
            temp_text_buffer = ""
            for part in self.content:
                if isinstance(part, TextPart):
                    temp_text_buffer += part.text
                else:
                    if temp_text_buffer:
                        new_content.append(TextPart(type='text', text=temp_text_buffer))
                        temp_text_buffer = ""
                    new_content.append(part)
            if temp_text_buffer:
                new_content.append(TextPart(type='text', text=temp_text_buffer))
        
        # 赋值回 self.content，确保它总是 List[ContentPart]
        # Pydantic v2允许在after验证器中修改self
        self.content = new_content
        return self


class ChatCompletionRequest(BaseModel):
    """ x """
    model: str  # The model name you want your service to expose
    messages: List[ChatMessage]
    user_id: str = Field(default=1)
    topic: str = Field(default="生活与体验")
    temperature: Optional[float] = 0.7
    top_p: Optional[float] = 1.0
    n: Optional[int] = 1 # How many chat completion choices to generate for each input message.
    stream: Optional[bool] = False
    stop: Optional[Union[str, List[str]]] = None
    max_tokens: Optional[int] = 2048 # Adjusted default for flexibility
    presence_penalty: Optional[float] = 0.0
    frequency_penalty: Optional[float] = 0.0
    logit_bias: Optional[Dict[str, float]] = None
    user: Optional[str] = None
    # Add other parameters if your model supports them (e.g., seed, tool_choice)


class ChatCompletionRequestDeep(BaseModel):
    """ x """
    messages: List[ChatMessage]
    user_id: str = Field(default=1)
    topic: str = Field(default="生活与体验")

    # Add other parameters if your model supports them (e.g., seed, tool_choice)


# --- Response Models (Non-Streaming) ---

class ChatCompletionMessage(BaseModel):
    """ x """
    role: Literal["assistant", "tool"] # Usually assistant
    content: Optional[str] = None
    # tool_calls: Optional[...]

class Choice(BaseModel):
    """ x """
    index: int
    message: ChatCompletionMessage
    finish_reason: Optional[Literal["stop", "length", "tool_calls",
                                    "content_filter", "function_call"]] = "stop"
    # logprobs: Optional[...]

class UsageInfo(BaseModel):
    """ x """
    prompt_tokens: int = 0 # You might need to implement token counting
    completion_tokens: int = 0
    total_tokens: int = 0

class ChatCompletionResponse(BaseModel):
    """ x """
    id: str = Field(default_factory=lambda: f"chatcmpl-{uuid.uuid4().hex}")
    object: Literal["chat.completion"] = "chat.completion"
    created: int = Field(default_factory=lambda: int(time.time()))
    model: str # Echo back the model requested or the one used
    choices: List[Choice]
    usage: UsageInfo = Field(default_factory=UsageInfo)
    # system_fingerprint: Optional[str] = None

# --- Response Models (Streaming) ---

class DeltaMessage(BaseModel):
    """ x """
    role: Optional[Literal["system", "user", "assistant", "tool"]] = None
    content: Optional[str] = None
    # tool_calls: Optional[...]

class ChunkChoice(BaseModel):
    """ x """
    index: int
    delta: DeltaMessage
    finish_reason: Optional[Literal["stop", "length", "tool_calls",
                                    "content_filter", "function_call"]] = None
    # logprobs: Optional[...]

class ChatCompletionChunkResponse(BaseModel):
    """ x """
    id: str = Field(default_factory=lambda: f"chatcmpl-{uuid.uuid4().hex}")
    object: Literal["chat.completion.chunk"] = "chat.completion.chunk"
    created: int = Field(default_factory=lambda: int(time.time()))
    model: str
    choices: List[ChunkChoice]
    # system_fingerprint: Optional[str] = None
    # usage: Optional[UsageInfo] = None

# --- (Optional) Add other OpenAI-like endpoints if needed ---
# For example, /v1/models to list available models
class ModelCard(BaseModel):
    """ x """
    id: str
    object: Literal["model"] = "model"
    created: int = Field(default_factory=lambda: int(time.time()))
    owned_by: str = "zhaoxuefeng" # Customize as needed

class ModelList(BaseModel):
    """ x """
    object: Literal["list"] = "list"
    data: List[ModelCard] = []


# server
from fastapi import FastAPI, HTTPException
from fastapi.middleware.cors import CORSMiddleware
from digital_life.core import MemoryCardManager
from digital_life.core import DigitalAvatar
from digital_life.server.router.digital_avatar import router as avatar_router
from digital_life.server.router.memory_card import router as memory_card_router
from digital_life.server.router.recommend.recommended import router as recommended_router
from digital_life.server.router.biography import router as biography_router
# from digital_life.server.router.prompt import router as prompt_router
from digital_life.server.router.chat import router as chat_router
from digital_life.models import LifeTopicScoreRequest, ScoreRequest, UseroverviewRequests, UserRelationshipExtractionRequest
from pro_craft.server.router.prompt import create_router
from digital_life import logger

import os
database_url = os.getenv("database_url")

                             
prompt_router = create_router(database_url=database_url,
              slave_database_url=database_url + "-test",
              model_name="doubao-1-5-pro-256k-250115",
              logger=logger)

prompt_inner_router = create_router(database_url=database_url,
              slave_database_url=database_url + "-test-inner",
              model_name="doubao-1-5-pro-256k-250115",
              logger=logger)
app = FastAPI(
    title="LLM Service",
    description="Provides an OpenAI-compatible API for custom large language models.",
    version="1.0.1",
)

# --- Configure CORS ---
origins = [
    "*", 
]

app.add_middleware(
    CORSMiddleware,
    allow_origins=origins,  # Specifies the allowed origins
    allow_credentials=True,  # Allows cookies/authorization headers
    allow_methods=["*"],  # Allows all methods (GET, POST, OPTIONS, etc.)
    allow_headers=["*"],  # Allows all headers (Content-Type, Authorization, etc.)
)
# --- End CORS Configuration ---


da = DigitalAvatar(inference_save_case = False,model_name = "doubao-1-5-pro-256k-250115")

app.include_router(avatar_router, prefix="/digital_avatar")
app.include_router(memory_card_router, prefix="/memory_card")
app.include_router(recommended_router, prefix="/recommended")
app.include_router(prompt_router, prefix="/prompt")
app.include_router(prompt_inner_router, prefix="/prompt_inner")
app.include_router(chat_router, prefix="/v1")
app.include_router(biography_router)

@app.get("/")
async def root():
    """server run"""
    envs = {
        "host":os.getenv("host"),
        "port":os.getenv("port"),
        "similarity_top_k":os.getenv("similarity_top_k"),
        "similarity_cutoff":os.getenv("similarity_cutoff"),
        "collection_name":os.getenv("collection_name"),
        "api_key":os.getenv("api_key"),
        "model_name":os.getenv("model_name"),
        "llm_model_name":os.getenv("llm_model_name"),
        "llm_api_key":os.getenv("llm_api_key"),
        "recommended_biographies_cache_max_leng":os.getenv("recommended_biographies_cache_max_leng"),
        "recommended_cache_max_leng":os.getenv("recommended_cache_max_leng"),
        "user_callback_url":os.getenv("user_callback_url"),
        "card_weight":os.getenv("card_weight"),
    }

    return {"message": "LLM Service is running.",
            "envs":envs}


@app.post("/life_topic_score")
async def life_topic_score_server(request: LifeTopicScoreRequest):
    try:
        result = MemoryCardManager.get_score(
            S=request.S_list,
            total_score=request.total_score,
            epsilon=request.epsilon,
            K=request.K,
        )
        return {
            "message": "Life topic score calculated successfully",
            "result": int(result),
        }
    except ValueError as e:
        raise HTTPException(status_code=400, detail=str(e))
    except Exception as e:
        raise HTTPException(
            status_code=500, detail=f"An unexpected error occurred: {str(e)}"
        )


@app.post("/life_aggregate_scheduling_score")
async def life_aggregate_scheduling_score_server(request: ScoreRequest):
    try:
        result = MemoryCardManager.get_score_overall(
            request.S_list,
            total_score=request.total_score,
            epsilon=request.epsilon,
            K=request.K,
        )
        return {
            "message": "life aggregate scheduling score successfully",
            "result": result,
        }
    except ValueError as e:
        raise HTTPException(status_code=400, detail=str(e))
    except Exception as e:
        raise HTTPException(
            status_code=500, detail=f"An unexpected error occurred: {str(e)}"
        )



@app.post("/user_overview")
async def user_overview_server(request: UseroverviewRequests):
    """
    用户概述
    """
    memory_cards = request.model_dump()["memory_cards"]
    result = await da.auser_overview(
        action = request.action,
        old_overview=request.old_overview, 
        memory_cards=memory_cards
    )  # 包裹的内核函数

    return {"overview": result}


@app.post("/user_relationship_extraction", description="用户关系提取")
async def user_relationship_extraction_server(
    request: UserRelationshipExtractionRequest,
):
    result = await da.auser_relationship_extraction(text=request.text)
    return {"relation": result}



if __name__ == "__main__":
    import argparse
    import uvicorn

    default = 8007
    
    parser = argparse.ArgumentParser(
        description="Start a simple HTTP server similar to http.server."
    )
    parser.add_argument(
        "port",
        metavar="PORT",
        type=int,
        nargs="?",  # 端口是可选的
        default=default,
        help=f"Specify alternate port [default: {default}]",
    )
    # 创建一个互斥组用于环境选择
    group = parser.add_mutually_exclusive_group()

    # 添加 --dev 选项
    group.add_argument(
        "--dev",
        action="store_true",  # 当存在 --dev 时，该值为 True
        help="Run in development mode (default).",
    )

    # 添加 --prod 选项
    group.add_argument(
        "--prod",
        action="store_true",  # 当存在 --prod 时，该值为 True
        help="Run in production mode.",
    )
    args = parser.parse_args()

    if args.prod:
        env = "prod"
    else:
        # 如果 --prod 不存在，默认就是 dev
        env = "dev"

    port = args.port

    if env == "dev":
        port += 100
        reload = True
        app_import_string = (
            f"{__package__}.__main__:app"  # <--- 关键修改：传递导入字符串
        )
    elif env == "prod":
        reload = False
        app_import_string = app
    else:
        reload = False
        app_import_string = app

    # 使用 uvicorn.run() 来启动服务器
    # 参数对应于命令行选项
    uvicorn.run(
        app_import_string, host="0.0.0.0", port=port, reload=reload  # 启用热重载
    )



# 1 日志不打在server中 不打在工具中, 只打在core 中

import math
import asyncio
from pro_craft import AsyncIntel
from pro_craft.utils import create_async_session
from digital_life.utils import memoryCards2str
from digital_life.models import MemoryCardGenerate,MemoryCardGenerateEasy, MemoryCard2, MemoryCard, MemoryCardScore, MemoryCards, Document, Chapter, MemoryCardsGenerate
from digital_life import super_log
from datetime import datetime
from typing import Dict, Any, Optional, List
from pydantic import BaseModel, Field, model_validator, field_validator, RootModel
import re

class MemoryCardManager:
    def __init__(self,inference_save_case = False,model_name = "doubao-1-5-pro-256k-250115"):
        self.inters = AsyncIntel(model_name = model_name)
        self.inference_save_case = inference_save_case

    @staticmethod
    def get_score_overall(
        S: list[int], total_score: int = 0, epsilon: float = 0.001, K: float = 0.8
    ) -> float:
        """
        计算 y = sqrt(1/600 * x) 的值。
        计算人生总进度
        """
        x = sum(S)
        
        S_r = [math.sqrt((1/101) * i)/6 for i in S]
        return sum(S_r)

        # return math.sqrt((1/601) * x)  * 100

    @staticmethod
    def get_score(
        S: list[int], total_score: int = 0, epsilon: float = 0.001, K: float = 0.01
    ) -> float:
        # 人生主题分值计算
        # 一个根据 列表分数 计算总分数的方法 如[1,4,5,7,1,5] 其中元素是 1-10 的整数

        # 一个非常小的正数，确保0分也有微弱贡献，100分也不是完美1
        # 调整系数，0 < K <= 1。K越大，总分增长越快。

        for score in S:
            # 1. 标准化每个分数到 (0, 1) 区间
            normalized_score = (score + epsilon) / (10 + epsilon)

            # 2. 更新总分
            # 每次增加的是“距离满分的剩余空间”的一个比例
            total_score = total_score + (100 - total_score) * normalized_score * K

            # 确保不会因为浮点数精度问题略微超过100，虽然理论上不会
            if total_score >= 100 - 1e-9:  # 留一点点余地，避免浮点数误差导致判断为100
                total_score = 100 - 1e-9  # 强制设置一个非常接近100但不等于100的值
                break  # 如果已经非常接近100，可以提前终止

        return total_score

    async def ascore_from_memory_card(self, memory_cards: list[str]) -> list[int]:
        # 正式运行 0088
        tasks = []
        for memory_card in memory_cards:
            tasks.append(
                self.inters.intellect_format(
                    input_data=memory_card,
                    prompt_id = "memorycard-score",
                    version = None,
                    inference_save_case=self.inference_save_case,
                    OutputFormat = MemoryCardScore,
                )
            )
        result = await asyncio.gather(*tasks, return_exceptions=False)
        return result

    async def amemory_card_merge(self, memory_cards: list[str]):
        # 0089
        memoryCards_str, memoryCards_time_str = memoryCards2str(memory_cards)
        result_1 = await self.inters.intellect_format(
            input_data=memoryCards_str + "\n 各记忆卡片的时间" + memoryCards_time_str,
            prompt_id = "memorycard-merge",
            version = None,
            inference_save_case=self.inference_save_case,
            OutputFormat = MemoryCard2,
        )
        return result_1

    async def amemory_card_polish(self, memory_card: dict) -> dict:
        # 0090
        result = await self.inters.intellect_format(
            input_data="\n记忆卡片标题: "+ memory_card["title"]+ "\n记忆卡片内容: " + memory_card["content"] + "\n记忆卡片发生时间: " + memory_card["time"],
            prompt_id = "memorycard-polish",
            version = None,
            inference_save_case=self.inference_save_case,
            OutputFormat = MemoryCard,
        )
        result.update({"time": ""})
        return result


    async def agenerate_memory_card(self, chat_history_str: str, weight: int = 1000):
        """
        0093 聊天历史生成记忆卡片-memory_card_system_prompt
        0094 聊天历史生成记忆卡片-time_prompt
        """
        number_ = len(chat_history_str) // weight + 1
        result_dict = await self.inters.intellect_format(
            input_data = f"建议输出卡片数量:  {number_} 个记忆卡片" + chat_history_str,
            prompt_id = "memorycard-generate-content",
            version = None,
            inference_save_case=self.inference_save_case,
            OutputFormat = Document,
            ExtraFormats=[Chapter],
        )
        chapters = result_dict["chapters"]


        class MemoryCardGenerate2(BaseModel):
            title: str = Field(..., description="标题",min_length=1, max_length=30)
            content: str = Field(..., description="内容",min_length=1,max_length=1000)
            time: str = Field(..., description="日期格式,YYYY年MM月DD日,其中YYYY可以是4位数字或4个下划线,MM可以是2位数字或2个--,DD可以是2位数字或2个--。年龄范围 输出对应的文字描述, 比如:而立, 不惑")
            score: int = Field(..., description="卡片得分", ge=0, le=10)
            tag: str = Field(..., description="标签1",max_length=4)
            topic: int = Field(..., description="主题1-5",ge=0, le=5)

            @field_validator('time')
            @classmethod
            def validate_time_format(cls, v: str) -> str:
                combined_regex = r"^(?:(\d{4}|-{4}|-{2})年(\d{2}|-{2})月(\d{2}|-{2})日)"
                match = re.match(combined_regex, v)
                if match:
                    return v
                elif v in ["稚龄","少年","弱冠","而立","不惑","知天命","耳顺","古稀","耄耋","鲐背","期颐"]:
                    return v
                else:
                    raise ValueError("时间无效")
        
        class TimeCheck(BaseModel):
            time: str = Field(...,description="")
            @field_validator('time')
            @classmethod
            def validate_time_format(cls, v: str) -> str:
                combined_regex = r"^(?:(\d{4}|-{4}|-{2})年(\d{2}|-{2})月(\d{2}|-{2})日)"
                match = re.match(combined_regex, v)
                if match:
                    return v
                elif v in ["稚龄","少年","弱冠","而立","不惑","知天命","耳顺","古稀","耄耋","鲐背","期颐"]:
                    return v
                else:
                    raise ValueError("时间无效")


        time_dicts = await self.inters.intellect_formats(
            input_datas=[f"# chat_history: {chat_history_str} # chapter:" + chapter.get("content") + f"当前时间为{datetime.now()}" for chapter in chapters],
            prompt_id = "memorycard-format",
            version = None,
            inference_save_case=self.inference_save_case,
            OutputFormat = MemoryCardGenerate2,
        )

        doc = {"稚龄":"0到10岁",
        "少年":"11到20岁",
        "弱冠":"21到30岁",
        "而立":"31到40岁",
        "不惑":"41到50岁",
        "知天命":"51到60岁",
        "耳顺":"61到70岁",
        "古稀":"71到80岁",
        "耄耋":"81到90岁",
        "鲐背":"91到100岁",
        "期颐":"101到110岁"}

        time_dicts_time = [time_dict.get('time') for time_dict in time_dicts]
        for i,time_dict in enumerate(time_dicts):
            llm_result_time = time_dicts_time[i]
            try:
                TimeCheck(time = llm_result_time)
            except Exception as e:
                super_log(f"Error{e}",'error')
                llm_result_time = "----年--月--日"

            if "月" in llm_result_time:
                # 一定是 --一年--月--日
                time_ = llm_result_time.replace('-','1')
                try:
                    date_obj = datetime.strptime(time_, "%Y年%m月%d日")
                    target_date = datetime.now()
                    if date_obj > target_date:
                        llm_result_time = "----年--月--日"

                except Exception as e:
                    llm_result_time = "----年--月--日"

            xx = doc.get(llm_result_time,llm_result_time)
            time_dict.update({"time":xx})

        super_log(time_dicts,"generate_memory_card-time_dicts")
        if not len(time_dicts) == len([f"# chat_history: {chat_history_str} # chapter:" + chapter.get("content") for chapter in chapters]):
            chapters = chapters[:1]
        for i,chapter in enumerate(chapters):
            chapter.update(time_dicts[i])
        
        for chapter in chapters:
            try:
                MemoryCardsGenerate(memory_cards=[chapter])
            except Exception as e:
                super_log(f"{e}",'agenerate_memory_card Error')
                chapter.update({"time":"----年--月--日"})
                
        return chapters







    async def agenerate_memory_card_by_text(self, chat_history_str: str, weight: int = 1000):
        return await self.agenerate_memory_card(chat_history_str = chat_history_str,weight = weight)
    




        





from fastapi import APIRouter, HTTPException, status
from typing import Dict, Any
from digital_life.core import BiographyGenerate
from digital_life.models import BiographyRequest, BiographyResult
from digital_life import logger
from digital_life import super_log
import asyncio
import httpx
import uuid
from digital_life.core.recommended.redis_ import get_value


bg = BiographyGenerate(inference_save_case = True,
                       model_name = "doubao-1-5-pro-256k-250115")

router = APIRouter(tags=["biography"])

# 免费版传记优化
@router.post("/generate_biography_free", summary="提交传记生成请求")
async def generate_biography(request: BiographyRequest):
    """
    提交一个传记生成请求。

    此接口会立即返回一个任务ID，客户端可以使用此ID查询生成进度和结果。
    实际的生成过程会在后台异步执行。
    """
    try:
        memory_cards = request.model_dump()["memory_cards"]
        result = await bg.agenerate_biography_free(
            user_name=request.user_name,
            memory_cards=memory_cards,
            vitae=request.vitae,
        )
        return result
    except Exception as e:
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=f"BiographyGenerate.agenerate_biography_free Error : {e}",
        )

@router.post(
    "/generate_biography", response_model=BiographyResult, summary="提交传记生成请求"
)
async def generate_biography(request: BiographyRequest):
    """
    提交一个传记生成请求。

    此接口会立即返回一个任务ID，客户端可以使用此ID查询生成进度和结果。
    实际的生成过程会在后台异步执行。
    """
    try:

        task_id = str(uuid.uuid4())
        memory_cards = request.model_dump()["memory_cards"]
        vitae = request.vitae
        user_name = request.user_name

        asyncio.create_task(bg._generate_biography(task_id, 
                                                   memory_cards = memory_cards,
                                                   vitae = vitae,
                                                   user_name = user_name
                                                   ))
        return BiographyResult(task_id=task_id, status="PENDING", progress=0.0)

    except Exception as e:
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=f"{task_id},task_store.create Error : {e}",
        )
@router.get(
    "/get_biography_result/{task_id}",
    response_model=BiographyResult,
    summary="查询传记生成结果",
)
async def get_biography_result(task_id: str):
    """
    根据任务ID查询传记生成任务的状态和结果。
    """
    try:
        task_info = get_value(bg.biograph_redis,task_id)
        if not task_info:
            raise HTTPException(
                status_code=404, detail=f"Task with ID '{task_id}' not found."
            )
    except Exception as e:
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=f"task_store.get Error : {e}",
        )

    return BiographyResult(
        task_id=task_info["task_id"],
        status=task_info["status"],
        biography_title=task_info.get("biography_title", "未知"),
        biography_brief=task_info.get("biography_brief", "未知"),
        biography_json=task_info.get("biography_json", {}),
        biography_name=task_info.get("biography_name", []),
        biography_place=task_info.get("biography_place", []),
        error_message=task_info.get("error_message"),
        progress=task_info.get("progress", 0.0),
    )

